#include <syscall.h>

#define __KERNEL__ 
#undef asmlinkage
#define asmlinkage
#include <sys/msg.h>

int
msgsnd (int msqid, struct msgbuf *msgp, int msgsz, int msgflg)
{
    return __ipc (MSGSND, msqid, msgsz, msgflg,  msgp);
}
