#include <gnu-stabs.h>
#include <sys/stat.h>

function_alias(stat, prev_stat, int, (name, buf),
		DEFUN(stat, (name, buf),
		CONST char *name AND struct stat *buf))

function_alias(fstat, prev_fstat, int, (fd, buf),
		DEFUN(fstat, (fd, buf),
		int fd AND struct stat *buf))

function_alias(mknod, prev_mknod, int, (name, mode, dev),
		DEFUN(fstat, (name, mode, dev),
		CONST char *name AND mode_t mode AND dev_t dev))

function_alias(lstat, prev_lstat, int, (name, buf),
		DEFUN(lstat, (name, buf),
		CONST char *name AND struct stat *buf))

