/* Copyright (C) 1991, 1992 Free Software Foundation, Inc.
This file is part of the GNU C Library.

The GNU C Library is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public License as
published by the Free Software Foundation; either version 2 of the
License, or (at your option) any later version.

The GNU C Library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Library General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with the GNU C Library; see the file COPYING.LIB.  If
not, write to the Free Software Foundation, Inc., 675 Mass Ave,
Cambridge, MA 02139, USA.  */

#include <syscall.h>
#if 0
#include <sys/stat.h>

static inline
_syscall2(int, prev_stat, const char *, name, struct stat *, buf);

int
__stat(const char *name, struct stat *buf) {
	return prev_stat(name, buf);
}
#else
/*
   We fake it because if we #include <sys/stat.h> we get inlined functions,
   but we need real functions for our PLT. Instead we pass void * pointers.
*/

static inline
_syscall2(int, prev_stat, const char *, name, void *, buf);

int
__stat(const char *name, void *buf) {
	return prev_stat(name, buf);
}

#endif
