/* autoconf.h.  Generated automatically by configure.  */
/* autoconf.h.in.  Generated automatically from configure.in by autoheader.  */

/* Define if using alloca.c.  */
/* #undef C_ALLOCA */

/* Define to empty if the keyword does not work.  */
/* #undef const */

/* Define to one of _getb67, GETB67, getb67 for Cray-2 and Cray-YMP systems.
   This function is required for alloca.c support on those systems.  */
/* #undef CRAY_STACKSEG_END */

/* Define if you have alloca.h and it should be used (not Ultrix).  */
#define HAVE_ALLOCA_H 1

/* Define if your struct stat has st_blksize.  */
#define HAVE_ST_BLKSIZE 1

/* Define if int is 16 bits instead of 32.  */
/* #undef INT_16_BITS */

/* Define if long int is 64 bits.  */
/* #undef LONG_64_BITS */

/* Define to `long' if <sys/types.h> doesn't define.  */
/* #undef off_t */

/* If using the C implementation of alloca, define if you know the
   direction of stack growth for your system; otherwise it will be
   automatically deduced at run-time.
	STACK_DIRECTION > 0 => grows toward higher addresses
	STACK_DIRECTION < 0 => grows toward lower addresses
	STACK_DIRECTION = 0 => direction of growth unknown
 */
/* #undef STACK_DIRECTION */

/* Define if your processor stores words with the most significant
   byte first (like Motorola and SPARC, unlike Intel and VAX).  */
/* #undef WORDS_BIGENDIAN */

/* Define if you have bcopy.  */
#define HAVE_BCOPY 1

/* Define if you have flock.  */
#define HAVE_FLOCK 1

/* Define if you have fsync.  */
#define HAVE_FSYNC 1

/* Define if you have ftruncate.  */
#define HAVE_FTRUNCATE 1

/* Define if you have rename.  */
#define HAVE_RENAME 1

/* Define if you have the <fcntl.h> header file.  */
#define HAVE_FCNTL_H 1

/* Define if you have the <memory.h> header file.  */
#define HAVE_MEMORY_H 1

/* Define if you have the <stdlib.h> header file.  */
#define HAVE_STDLIB_H 1

/* Define if you have the <string.h> header file.  */
#define HAVE_STRING_H 1

/* Define if you have the <sys/file.h> header file.  */
#define HAVE_SYS_FILE_H 1

/* Define if you have the <sys/types.h> header file.  */
#define HAVE_SYS_TYPES_H 1

/* Define if you have the <unistd.h> header file.  */
#define HAVE_UNISTD_H 1

/* Define if you have the dbm library (-ldbm).  */
#define HAVE_LIBDBM 1

/* Define if you have the ndbm library (-lndbm).  */
/* #undef HAVE_LIBNDBM */
